/****************************************************************************
 *
 * MODULE:             Intelligent Peripheral Interface
 *
 * COMPONENT:          $RCSfile: $
 *
 * VERSION:            $Name: $
 *
 * REVISION:           $Revision: $
 *
 * DATED:              $Date: $
 *
 * STATUS:             $State: $
 *
 * AUTHOR:             Mozammul Ahmed
 *
 * DESCRIPTION:
 * Example of an IP mode interface on the host side.
 *
 * LAST MODIFIED BY:   $Author:  $
 *                     $Modtime: $
 *
 *
 ****************************************************************************
 *
 *  (c) Copyright 2000 JENNIC Ltd
 *
 ****************************************************************************/

/****************************************************************************/
/***        Include files                                                 ***/
/****************************************************************************/

#include <jendefs.h>
#include <AppHardwareApi.h>
#include "ip.h"
#include "debug.h"

/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/

#define IP_STATUS_TXQ_MASK      ((uint8)0x02)
#define IP_STATUS_RXRDY_MASK    ((uint8)0x01)

/****************************************************************************/
/***        Type Definitions                                              ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Function Prototypes                                     ***/
/****************************************************************************/

PRIVATE uint8 u8FindMax (uint8 u8Rx, uint8 u8Tx);

/****************************************************************************/
/***        Exported Variables                                            ***/
/****************************************************************************/

/****************************************************************************/
/***        Local Variables                                               ***/
/****************************************************************************/

PRIVATE uint8   au8RxMsgBuffer[E_AHI_IP_MAX_MSG_SIZE*4];
PRIVATE uint8   au8TxMsgBuffer[E_AHI_IP_MAX_MSG_SIZE*4];

PRIVATE uint8   u8RxMsgLength;
PRIVATE uint8   u8TxMsgLength;

PRIVATE uint8   u8SlaveStatus; /* status byte read from slave */
PRIVATE uint8   u8MasterStatus; /* status byte to transfer to slave */

PRIVATE PR_IP_APPCALLBACK prIpAppCallback = NULL;

/****************************************************************************/
/***        Exported Functions                                            ***/
/****************************************************************************/

/****************************************************************************
 *
 * NAME:       vIP_Init
 *
 * DESCRIPTION:
 * Initialise IP host interface.
 *
 * PARAMETERS: None
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PUBLIC void vIP_Init(void)
{
    /* initialise local variables */
    u8SlaveStatus = 0;
    u8MasterStatus = IP_STATUS_RXRDY_MASK;
    u8RxMsgLength = 0;
    u8TxMsgLength = 0;

    /* configure SPI interface */
    vAHI_SpiConfigure(1, E_AHI_SPIM_MSB_FIRST, 0,
                      0, 1, E_AHI_SPIM_INT_DISABLE,
                      E_AHI_SPIM_AUTOSLAVE_DSABL);

}

/****************************************************************************
 *
 * NAME:       bIP_MsgSend
 *
 * DESCRIPTION:
 * Prepare message for sending, if data is not whole number of words then
 * pads it out with zero.
 *
 * PARAMETERS: Name              RW  Usage
 *             u8Length          R   number of words to send
 *             pau8Data          R   pointer to data to send
 *                                   (assumed LSB first)
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PUBLIC bool_t bIP_MsgPrepare(uint8 u8Length, uint8 *pau8Data)
{
    uint8 i;

    /* check length */
    if (u8Length > E_AHI_IP_MAX_MSG_SIZE)
    {
        return FALSE;
    }

    /* set tx length field */
    u8TxMsgLength = u8Length + 1; // the first entry is data length in DWORD

    /* set status */
    vDebug("TxLngth ");
    vDisplayHex((uint32) u8Length);
    vDebug("\r\n");

    vDebug("MstrStat ");
    vDisplayHex((uint32) u8MasterStatus);
    vDebug("\r\n");

    /* copy data to buffer */
    uint32 *u32p = (uint32*)au8TxMsgBuffer;
    uint32 *u32q = (uint32*)pau8Data;
    u32p[0] = u8Length;
    if (pau8Data) {
		for (i=1; i<= u8Length; i++)
		{
			u32p[i] = u32q[i - 1];
		}
    } else {
		for (i=1; i<=u8Length; i++)
		{
			u32p[i]++;
		}
    }

    return TRUE;

}

/****************************************************************************
 *
 * NAME:       vIP_RegisterCallback
 *
 * DESCRIPTION:
 * Register Msg Complete callback
 *
 * PARAMETERS: Name              RW  Usage
 *             prIpCallback      R   Pointer to Callback routine
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PUBLIC void vIP_RegisterCallback(PR_IP_APPCALLBACK prIpCallback)
{
    prIpAppCallback = prIpCallback;
}

/****************************************************************************
 *
 * NAME:       vIP_Communicate
 *
 * DESCRIPTION:
 * Blocking call to initiate tx/rx of data, if there's any data to send, will
 * ensure it gets sent, if there's any to come, will ensure it gets through
 * and will call the msg complete callback.
 *
 * PARAMETERS: None
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
PUBLIC void vIP_Communicate(void)
{
    uint8 u8Max, u8RxLength, u8TxLength;
    uint8 i, j;
    uint16 u16Out, u16In;
    uint32 u32DataWord;
    bool_t bSendData = FALSE;
    bool_t bRecvData = FALSE;

    vDebug("CommStrt\r\n");

    /* select slave 1*/
    //vAHI_SpiSelect(E_AHI_SPIM_SLAVE_ENBLE_1);
    vAHI_SpiSelect(2); // Select SPI_SLAVE #1 (DIO0, BIT1)

    /* transfer status */
    u8MasterStatus = IP_STATUS_RXRDY_MASK | IP_STATUS_TXQ_MASK; // TRANSMIT AND RECV
    u16Out = 0xFF00 & (u8MasterStatus << 8);

    vAHI_SpiStartTransfer(15,u16Out);
    vAHI_SpiWaitBusy();

    /* read slave status */
    u16In = u16AHI_SpiReadTransfer16();
    u8SlaveStatus = (uint8)u16In & 0xFF;

    vDebug("SlvStat ");
    vDisplayHex((uint32) u8SlaveStatus);
    vDebug("\r\n");

    /* transfer length */
    vAHI_SpiStartTransfer(7,u8TxMsgLength);
    vAHI_SpiWaitBusy();

    /* read length */
    u8RxMsgLength = u8AHI_SpiReadTransfer8();

    if ( (u8SlaveStatus & IP_STATUS_RXRDY_MASK) &&
         (u8MasterStatus & IP_STATUS_TXQ_MASK) )
    {
        /* slave ready to receive and we're ready to send */
        bSendData = TRUE;
    }
    else
    {
        u8TxMsgLength = 0;
    }

    if ( (u8MasterStatus & IP_STATUS_RXRDY_MASK) &&
         (u8SlaveStatus & IP_STATUS_TXQ_MASK) )
    {
        /* slave ready to send and we're ready to receive */
        bRecvData = TRUE;
    }
    else
    {
        u8RxMsgLength = 0;
    }

    u8Max = u8FindMax(u8RxMsgLength, u8TxMsgLength);
    u8RxLength = u8RxMsgLength;
    u8TxLength = u8TxMsgLength;

    vDebug("MstrLngth ");
    vDisplayHex((uint32) u8TxMsgLength);
    vDebug("\r\n");

    vDebug("SlvLngth ");
    vDisplayHex((uint32) u8RxMsgLength);
    vDebug("\r\n");

    vDebug("MaxLngth ");
    vDisplayHex((uint32) u8Max);
    vDebug("\r\n");

    for (i=0; i<u8Max; i++)
    {
        j = 4 * i;

        vDisplayHex((uint32) i);
        vDebug(" ");

        if (bSendData && (u8TxLength!=0))
        {
            /* slave ready to receive and we're ready to send */
            /* MSB first */
            u32DataWord = au8TxMsgBuffer[j+3];
            u32DataWord |= (au8TxMsgBuffer[j+2] << 8);
            u32DataWord |= (au8TxMsgBuffer[j+1] << 16);
            u32DataWord |= (au8TxMsgBuffer[j] << 24);

            vAHI_SpiStartTransfer(31,u32DataWord);

            vDebug("Snd ");
            vDisplayHex(u32DataWord);

            u8TxLength -= 1;
        }
        else
        {
            /* send padding */
            vAHI_SpiStartTransfer(31,0x0);
            vDebug("Snd 00000000");

        }

        vAHI_SpiWaitBusy();

        if (bRecvData && (u8RxLength!=0))
        {
            /* slave ready to send and we're ready to receive */
            u32DataWord = u32AHI_SpiReadTransfer32();

            /* MSB first */
            au8RxMsgBuffer[j] = (uint8) (0xFF & (u32DataWord >> 24) );
            au8RxMsgBuffer[j+1] = (uint8) (0xFF & (u32DataWord >> 16) );
            au8RxMsgBuffer[j+2] = (uint8) (0xFF & (u32DataWord >> 8) );
            au8RxMsgBuffer[j+3] = (uint8) (0xFF & u32DataWord);

            vDebug(" Rcv ");
            vDisplayHex(u32DataWord);

            u8RxLength -= 1;

        }
        vDebug("\r\n");
    }

    /* De-assert select */
    vAHI_SpiStop();
    vDebug("SpiStop\r\n");
}

/****************************************************************************/
/***        Local Functions                                               ***/
/****************************************************************************/

PRIVATE uint8 u8FindMax (uint8 u8Rx, uint8 u8Tx)
{
    if (u8Rx > E_AHI_IP_MAX_MSG_SIZE)
    {
        return u8Tx;
    }
    else if (u8Rx > u8Tx)
    {
        return u8Rx;
    }
    else
    {
        return u8Tx;
    }
}

/****************************************************************************/
/***        END OF FILE                                                   ***/
/****************************************************************************/
